package com.tr.tms.abstractionLayer;

import com.tr.tms.abstractionLayer.engine.Engine;
import com.tr.tms.abstractionLayer.engine.Engine.Format;
import com.tr.tms.abstractionLayer.engine.RdfEngine;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;
import java.util.Map;

/**
 * CalaisModelReader provides methods for creating CalaisModel.
 * <p/>
 * <P>This class supplies convenience methods, for CalaisModel initialization.
 * It is intended use is as a base class from which CalaisModel can be derived.</P>
 */
public class CalaisModelCreator {
    private static final Logger log = LoggerFactory.getLogger(CalaisModelCreator.class);

    /**
     * No-one can make instances of this.
     */
    private CalaisModelCreator() {
    }

    /**
     * receive the path of file contains RDF text, process it and return a CalaisModel
     *
     * @param fileName    - the path name of the file in the file system
     * @param inputFormat - the format is RDF or N3
     * @return CalaisModel
     */
    public static CalaisModel readFile(String fileName, Format inputFormat) throws IllegalArgumentException {
        log.debug("CalaisModel, readFile method, file name:" + fileName);
        FileInputStream inputStream = null;
        CalaisModel calaisModel = null;
        try {
            try {
                inputStream = new FileInputStream(fileName);
                calaisModel = readInputStream(inputStream, inputFormat);
            } finally {
                inputStream.close();
            }
        } catch (Exception e) {
            log.error("Failed to load the file:" + fileName, e);
            throw new IllegalArgumentException("Failed to load the file:" + fileName, e);
        }
        return calaisModel;
    }

    /**
     * receive RDF as string, process it and return a CalaisModel
     *
     * @param rdfText
     * @param inputFormat - the format is RDF or N3
     * @return CalaisModel
     */
    public static CalaisModel readText(String rdfText, Format inputFormat) throws IllegalArgumentException {

        log.debug("CalaisModel, readText method");
        Engine engine = null;
        //now we can start the engine to create our objects
        if (inputFormat.equals(Format.RDF)) {
            engine = new RdfEngine();
        }
        return engine.process(rdfText, inputFormat);

    }

    /**
     * receive RDF as input stream, process it and return a CalaisModel
     *
     * @param inputStream
     * @param inputFormat - the format is RDF or N3
     * @return CalaisModel
     */
    public static CalaisModel readInputStream(InputStream inputStream, Format inputFormat) throws IllegalArgumentException {
        log.debug("CalaisModel, readInputStream method");
        //now we can start the engine to create our object
        Engine engine = null;
        if (inputFormat.equals(Format.RDF) || inputFormat.equals(Format.N3)) {
            engine = new RdfEngine();
        }
        return engine.process(inputStream, inputFormat);
    }

}
