package com.tr.tms.abstractionLayer.engine;

import com.tr.tms.abstractionLayer.CalaisModel;
import com.tr.tms.abstractionLayer.CalaisObject;
import com.hp.hpl.jena.rdf.model.*;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.*;

public class RdfEngine implements Engine {
    private static Logger log = LoggerFactory.getLogger(RdfEngine.class);
    private static final String RDF_TYPE = "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";
    private List<String> errors = new ArrayList<String>();

    public RdfEngine() {
    }

    public List<String> getErrors() {
        return Collections.unmodifiableList(errors);
    }

    protected void onError(String message){
        onError(message,null);
    }

    protected void onError(String message,Throwable throwable){
        if (throwable!=null)
            log.error(message, throwable);
        else
            log.error(message);
        errors.add(message);
    }

    public CalaisModel process(InputStream inputStream, Format rdfFormat) throws IllegalArgumentException {
        if (inputStream == null) {
            onError("failed to create model calais due to the inputStream is empty");
            throw new IllegalArgumentException("failed to create model calais due to the inputStream is empty");
        }
        try {
            // read the RDF/XML inputStream
            Model model = createModelForInputStream(inputStream, rdfFormat);
            return process(model);
        } catch (Exception e) {
            onError("failed to create model calais", e);
            throw new IllegalArgumentException("failed to create model calais", e);
        }
    }

    public CalaisModel process(String rdfText, Format rdfFormat) throws IllegalArgumentException {
        if (rdfText == null || rdfText.length() == 0) {
            onError("failed to create model calais due to the string is empty");
            throw new IllegalArgumentException("failed to create model calais due to the string is empty");
        }
        try {
            // read the RDF/XML inputStream
            Model model = createModelForText(rdfText, rdfFormat);
            return process(model);
        } catch (Exception e) {
            onError("failed to create model calais", e);
            throw new IllegalArgumentException("failed to create model calais", e);
        }
    }

    protected Model createModelForInputStream(InputStream inputStream, Format rdfFormat) {
        Model model = ModelFactory.createDefaultModel();
        model.read(inputStream, null, rdfFormat.getValue());
        return model;
    }

    protected Model createModelForText(String rdfText, Format rdfFormat) throws Exception {
        InputStream inputStream;
        inputStream = new ByteArrayInputStream(rdfText.getBytes("UTF-8"));
        return createModelForInputStream(inputStream, rdfFormat);
    }

    private CalaisModel process(Model model) {
        Map<String, CalaisObject> classesMap = new HashMap<>();

        //1. Create the classes and insert them into the classesMap.
        createTheClasses(model, classesMap);

        //2. fill the classes with the data
        addDataToClasses(model, classesMap);

        //3.create the calaisModel and fill it with the data
        CalaisModel calaisModel = createCalaisModel(classesMap);

        return calaisModel;
    }


    /**
     * This method just create the classes and insert them into the 'classesMap'
     *
     * @param model
     */
    private void createTheClasses(Model model, Map<String, CalaisObject> classesMap) {
        // list the classes in the Model with the predicate rdf-syntax-ns#type
        StmtIterator iter = model.listStatements(new SimpleSelector(null, null, (RDFNode) null) {
            public boolean selects(Statement s) {
                return (predicate == null && s.getPredicate().getURI().equals(RDF_TYPE));
            }
        });

        //create the instance of the classes and insert them into the map
        while (iter.hasNext()) {
            Statement typeStatement = iter.nextStatement();  // get next statement
            //get the object - the class name
            String classType = typeStatement.getObject().toString();
            String classId = typeStatement.getSubject().toString();
            final CalaisObject calaisObject = new CalaisObject(classId, classType);
            classesMap.put(classId, calaisObject);
        }
    }

    /**
     * Fill the classes (from the classMap) with the data.
     *
     * @param model
     */
    private void addDataToClasses(Model model, Map<String, CalaisObject> classesMap) {
        //set all the properties
        StmtIterator iterator = model.listStatements();
        CalaisObject calaisObject = null;
        Statement statement = null;
        while (iterator.hasNext()) {
            try {
                // get next statement
                statement = iterator.nextStatement();
                //check whether the predicate is different than 'RDF_TYPE'
                Property predicate = statement.getPredicate();   // get the predicate
                if (!predicate.toString().equals(RDF_TYPE)) {
                    //extract the field name from the predicate
                    String fieldName = predicate.toString();
                    //extract the class from the subject via classesMap
                    String classKey = statement.getSubject().toString();
                    if (calaisObject == null || !calaisObject.getObjectId().equals(classKey)) {
                        calaisObject = classesMap.get(classKey);
                    }
                    if (calaisObject == null) {
                        onError("There isn't a class with the id:" + statement.getSubject().toString());
                        continue;
                    }
                    //extract the field value from the object
                    RDFNode rdfNode = statement.getObject();
                    if (rdfNode instanceof Resource) {
                        CalaisObject fieldValue = classesMap.get(rdfNode.toString());
                        if (fieldValue != null) {
                            calaisObject.addReference(fieldName, fieldValue);
                            fieldValue.addBackReference(fieldName, calaisObject);
                        } else {//it is a URI
                            calaisObject.addExternalURI(fieldName, rdfNode.toString());
                        }
                    }else{//it is a string
                        calaisObject.addLiteral(fieldName, rdfNode.toString());
                    }
                }
            }catch (Exception e){
                onError(e.getMessage(), e);
            }
        }
    }

    /**
     * create the calaisModel by the classesMap
     *
     * @param classesMap
     * @return
     */
    private CalaisModel createCalaisModel(Map<String, CalaisObject> classesMap) {
        CalaisModel calaisModel = new CalaisModel();
        for (CalaisObject calaisObject : classesMap.values()) {
            calaisModel.addCalaisObject(calaisObject);
        }
        return calaisModel;
    }


}

